"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const spawn = require("cross-spawn");
const moment = require("moment");
const path = require("path");
const webpack = require("webpack");
const build_intermediate_dependencies_1 = require("../components/build_intermediate_dependencies");
const init_project_1 = require("../components/init_project");
const copy_bundles_to_dist_1 = require("../components/project_deps/copy_bundles_to_dist");
const show_docker_commands_1 = require("../components/show_docker_commands");
const workbox_1 = require("../components/workbox");
const buildStatus_1 = require("../config/buildStatus");
const env_1 = require("../config/env");
const rearguard_1 = require("../config/rearguard");
const examples_1 = require("../meta/examples");
const frontEndDockerfile_1 = require("../meta/frontEndDockerfile");
const nginxConfig_1 = require("../meta/nginxConfig");
const get_stats_1 = require("../webpack/components/get_stats");
const webpack_config_dll_1 = require("../webpack/webpack.config.dll");
const webpack_config_lib_1 = require("../webpack/webpack.config.lib");
const webpack_config_main_1 = require("../webpack/webpack.config.main");
function build_node_lib() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(chalk_1.default.bold.blue(`[ TYPESCRIPT_COMPILE ][ START ]`));
        console.log("");
        const startTime = moment();
        const result = spawn.sync("tsc", [
            "--project",
            path.resolve(process.cwd(), "tsconfig.json"),
            "--rootDir",
            path.resolve(process.cwd(), "src"),
            "--outDir",
            path.resolve(process.cwd(), "lib"),
            "--module",
            "commonjs",
            "--declaration",
        ], {
            cwd: process.cwd(),
            encoding: "utf8",
            stdio: "inherit",
        });
        // ! Обработка сигнала.
        if (result.signal) {
            if (result.signal === "SIGKILL") {
                console.log(chalk_1.default.bold.red("The tsc failed because the process exited too early. " +
                    "This probably means the system ran out of memory or someone called `kill -9` on the process."));
                process.exit(1);
            }
            else if (result.signal === "SIGTERM") {
                console.log(chalk_1.default.bold.red("The tsc failed because the process exited too early. " +
                    "Someone might have called `kill` or `killall`, or the system could be shutting down."));
                process.exit(1);
            }
        }
        console.log("");
        console.log(chalk_1.default.bold.blue(`[ TYPESCRIPT_COMPILE ][ END ][ ${moment().diff(startTime, "milliseconds")} ms ]`));
        console.log("");
    });
}
function build() {
    return __awaiter(this, void 0, void 0, function* () {
        if (env_1.envConfig.has_dll) {
            console.log(chalk_1.default.bold.blue(`[ BUILD_DLL ][ START ]`));
            const startTime = moment();
            yield new Promise((resolve, reject) => {
                webpack(webpack_config_dll_1.dll_WP_config(env_1.envConfig, rearguard_1.rearguardConfig)).run((err, stats) => __awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        reject(err);
                    }
                    console.info(stats.toString(get_stats_1.get_stats(env_1.envConfig)));
                    resolve();
                }));
            });
            console.log("");
            console.log(chalk_1.default.bold.blue(`[ BUILD_DLL ][ END ][ ${moment().diff(startTime, "milliseconds")} ms ]`));
            console.log("");
        }
        if (env_1.envConfig.has_browser_lib) {
            console.log(chalk_1.default.bold.blue(`[ BUILD_LIBRARY ][ START ]`));
            const startTime = moment();
            yield new Promise((resolve, reject) => {
                webpack(webpack_config_lib_1.library_WP_config(env_1.envConfig, rearguard_1.rearguardConfig)).run((err, stats) => __awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        reject(err);
                    }
                    console.info(stats.toString(get_stats_1.get_stats(env_1.envConfig)));
                    resolve();
                }));
            });
            console.log("");
            console.log(chalk_1.default.bold.blue(`[ BUILD_LIBRARY ][ END ][ ${moment().diff(startTime, "milliseconds")} ms ]`));
            console.log("");
        }
        if (env_1.envConfig.is_application) {
            console.log(chalk_1.default.bold.blue(`[ BUILD_PROJECT ][ START ]`));
            const startTime = moment();
            // ! Docker config
            frontEndDockerfile_1.frontEndDockerfile.init(env_1.envConfig);
            // ! Nginx config
            nginxConfig_1.nginxConfig.init(env_1.envConfig);
            // ! Examples
            examples_1.examples.init(env_1.envConfig, true);
            // Сборка front-end проекта.
            yield new Promise((resolve, reject) => {
                webpack(webpack_config_main_1.main_WS_config(env_1.envConfig, rearguard_1.rearguardConfig)).run((err, stats) => __awaiter(this, void 0, void 0, function* () {
                    if (err) {
                        reject(err);
                    }
                    console.info(stats.toString(get_stats_1.get_stats(env_1.envConfig)));
                    console.log("");
                    yield copy_bundles_to_dist_1.copy_bundles_to_dist(env_1.envConfig);
                    yield workbox_1.create_workbox(env_1.envConfig);
                    resolve();
                }));
            });
            console.log("");
            console.log(chalk_1.default.bold.blue(`[ BUILD_PROJECT ][ END ][ ${moment().diff(startTime, "milliseconds")} ms ]`));
            console.log("");
            show_docker_commands_1.show_docker_commands(rearguard_1.rearguardConfig);
        }
    });
}
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        yield build_intermediate_dependencies_1.build_intermediate_dependencies(env_1.envConfig, rearguard_1.rearguardConfig);
        buildStatus_1.buildStatusConfig.start();
        yield init_project_1.initProject();
        if (env_1.envConfig.isBuildBoth) {
            process.env.NODE_ENV = "development";
            yield build();
            process.env.NODE_ENV = "production";
            yield build();
        }
        else {
            yield build();
        }
        if (env_1.envConfig.has_node_lib) {
            yield build_node_lib();
            if (!env_1.envConfig.has_dll && !env_1.envConfig.has_browser_lib && !env_1.envConfig.is_application) {
                buildStatus_1.buildStatusConfig.last_build_time = moment();
            }
        }
        buildStatus_1.buildStatusConfig.end();
    });
}
run();
