"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const child_process_2 = require("child_process");
const ora_1 = require("ora");
const semver = require("semver");
const util_1 = require("util");
const check_npm_1 = require("../components/check_npm");
const install_declared_deps_1 = require("../components/project_deps/install_declared_deps");
const install_dev_deps_1 = require("../components/project_deps/install_dev_deps");
const ordering_project_deps_1 = require("../components/project_deps/ordering_project_deps");
const sync_with_linked_modules_1 = require("../components/project_deps/sync_with_linked_modules");
const env_1 = require("../config/env");
const rearguard_1 = require("../config/rearguard");
function search(depName) {
    return __awaiter(this, void 0, void 0, function* () {
        return yield new Promise((resolve) => {
            let timeout;
            const spinner = ora_1.default(`Search ${depName} on npm registry`).start();
            const childProcess = child_process_1.exec(`npm search --json ${depName}`, { encoding: "utf8" }, (error, stdout) => {
                clearTimeout(timeout);
                if (!error) {
                    try {
                        const result = JSON.parse(stdout);
                        if (util_1.isArray(result) && result.length > 0) {
                            spinner.succeed(`${depName} is found`);
                            resolve(result);
                        }
                        else {
                            spinner.fail(`${depName} is not found`);
                            resolve();
                        }
                    }
                    catch (error) {
                        spinner.fail(`${depName} is not found`);
                        resolve();
                        console.error(error);
                    }
                }
                else {
                    spinner.fail(`${depName} is not found`);
                    resolve();
                }
                console.log("");
            });
            timeout = setTimeout(() => {
                spinner.fail(`${depName} is not found`);
                console.log("");
                childProcess.kill();
                resolve();
            }, 10000);
        });
    });
}
let needInstall = false;
function checker(envConfig, target, key) {
    return __awaiter(this, void 0, void 0, function* () {
        const { install_deps } = envConfig;
        const { pkg } = rearguard_1.rearguardConfig;
        if (utils_1.isObject(pkg[key])) {
            for (const targetDepName of target) {
                const curVersion = pkg[key][targetDepName];
                if (utils_1.isString(curVersion)) {
                    const result = yield search(targetDepName);
                    if (util_1.isArray(result) && result.length > 0) {
                        const version = result[0].version;
                        if (semver.valid(version) && semver.lt(curVersion, version)) {
                            console.log(chalk_1.default.white(`[ VERSION FOR: ${chalk_1.default.bold.yellow(targetDepName)} ` +
                                `ON NPM REGISTRY IS: ${chalk_1.default.bold.yellow(version)} ]`));
                            if (install_deps) {
                                console.log(chalk_1.default.white(`[ VERSION FOR: ${chalk_1.default.bold.yellow(targetDepName)} ` +
                                    `WILL UPDATE TO: ${chalk_1.default.bold.yellow(version)} ]`));
                                rearguard_1.rearguardConfig.pkg = { [key]: Object.assign({}, rearguard_1.rearguardConfig.pkg[key], { [targetDepName]: version }) };
                                needInstall = true;
                            }
                            console.log("");
                        }
                    }
                }
            }
        }
    });
}
function check_deps_on_npm(envConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        yield install_declared_deps_1.install_declared_deps(envConfig);
        yield install_dev_deps_1.install_dev_deps(envConfig);
        try {
            const { sync_project_deps } = rearguard_1.rearguardConfig;
            const { install_deps } = envConfig;
            if (sync_project_deps.length > 0) {
                /////////////////////
                //
                // START OF PROCEDURE
                //
                /////////////////////
                console.log(chalk_1.default.bold.blue(`=============CHECK_DEPS_ON_NPM==============`));
                console.log("");
                const npmIsAvailable = yield check_npm_1.check_npm();
                if (npmIsAvailable) {
                    yield checker(envConfig, sync_project_deps, "dependencies");
                    yield checker(envConfig, sync_project_deps, "devDependencies");
                    yield checker(envConfig, sync_project_deps, "peerDependencies");
                    if (install_deps && needInstall) {
                        console.log(chalk_1.default.white(`npm install`));
                        console.log("");
                        child_process_2.execSync(`npm install`, {
                            encoding: "utf8",
                            stdio: "inherit",
                        });
                        console.log("");
                        yield ordering_project_deps_1.ordering_project_deps(envConfig);
                        yield sync_with_linked_modules_1.sync_with_linked_modules(envConfig);
                    }
                }
                /////////////////////
                //
                // END OF PROCEDURE
                //
                /////////////////////
                console.log(chalk_1.default.bold.blue(`============================================`));
                console.log("");
            }
        }
        catch (error) {
            console.trace(chalk_1.default.red(`[ ERROR_MESSAGE: ${utils_1.getErrorMessage(error)} ]`));
            process.exit(1);
        }
    });
}
check_deps_on_npm(env_1.envConfig);
