"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const build_1 = require("../components/mono_repository/build");
const clear_1 = require("../components/mono_repository/clear");
const init_1 = require("../components/mono_repository/init");
const install_1 = require("../components/mono_repository/install");
const link_1 = require("../components/mono_repository/link");
const publish_1 = require("../components/mono_repository/publish");
const test_1 = require("../components/mono_repository/test");
const get_list_of_ordered_modules_1 = require("../components/project_deps/get_list_of_ordered_modules");
const env_1 = require("../config/env");
const monorepo_1 = require("../config/monorepo");
const RearguardConfig_1 = require("../config/rearguard/RearguardConfig");
// tslint:disable:variable-name
function monorepo() {
    return __awaiter(this, void 0, void 0, function* () {
        if (env_1.envConfig.is_mono_init) {
            yield init_1.init();
        }
        const root = path_1.resolve(process.cwd(), monorepo_1.monorepoConfig.modules);
        const dirs = fs_1.readdirSync(root).filter((value) => value.indexOf(".") === -1);
        const module_names = [];
        const module_map = new Map();
        for (const module_dir of dirs) {
            const module_path = path_1.resolve(root, module_dir);
            const { pkg } = new RearguardConfig_1.RearguardConfig(env_1.envConfig, path_1.resolve(module_path, "package.json"));
            module_map.set(pkg.name, module_path);
            module_names.push(pkg.name);
        }
        const oredered_modules = yield get_list_of_ordered_modules_1.get_list_of_ordered_modules(env_1.envConfig, root, module_names, module_map);
        const ordered_paths = oredered_modules
            .filter((N) => module_map.has(N)) // Пропускаются модули, только из директории packages,
            // так как могут быть внешние модули не входящие в состав монорепозитория.
            .map((N) => path_1.resolve(root, module_map.get(N) || ""));
        for (const module_path of ordered_paths) {
            if (!env_1.envConfig.is_mono_bootstrap && env_1.envConfig.is_mono_clear) {
                yield clear_1.clear(env_1.envConfig, module_path);
            }
            if (!env_1.envConfig.is_mono_bootstrap && env_1.envConfig.is_mono_install) {
                yield install_1.install(module_path);
            }
            if (!env_1.envConfig.is_mono_bootstrap && env_1.envConfig.is_mono_build) {
                yield build_1.build(module_path);
            }
            if (!env_1.envConfig.is_mono_bootstrap && env_1.envConfig.is_mono_link) {
                yield link_1.link(module_path);
            }
            if (env_1.envConfig.is_mono_bootstrap) {
                yield clear_1.clear(env_1.envConfig, module_path);
                yield install_1.install(module_path);
                yield build_1.build(module_path);
                yield link_1.link(module_path);
            }
            if (env_1.envConfig.is_mono_test) {
                yield test_1.test(module_path);
            }
            if (env_1.envConfig.is_mono_publish) {
                yield publish_1.publish(env_1.envConfig, module_path);
            }
        }
    });
}
monorepo();
// tslint:enable:variable-name
