"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const spawn = require("cross-spawn");
const build_intermediate_dependencies_1 = require("../components/build_intermediate_dependencies");
const install_declared_deps_1 = require("../components/project_deps/install_declared_deps");
const install_dev_deps_1 = require("../components/project_deps/install_dev_deps");
const ordering_project_deps_1 = require("../components/project_deps/ordering_project_deps");
const sync_with_linked_modules_1 = require("../components/project_deps/sync_with_linked_modules");
const env_1 = require("../config/env");
const prettier_1 = require("../config/prettier");
const rearguard_1 = require("../config/rearguard");
const tslint_1 = require("../config/tslint");
const typescript_1 = require("../config/typescript");
const dockerignore_1 = require("../meta/dockerignore");
const editorConfig_1 = require("../meta/editorConfig");
const gitignore_1 = require("../meta/gitignore");
const Npmrc_1 = require("../meta/Npmrc");
function start_node_server() {
    return __awaiter(this, void 0, void 0, function* () {
        yield build_intermediate_dependencies_1.build_intermediate_dependencies(env_1.envConfig, rearguard_1.rearguardConfig);
        // Config file
        typescript_1.typescriptConfig.init(true);
        tslint_1.tsLintConfig.init(true);
        editorConfig_1.editorConfig.init(env_1.envConfig, true);
        prettier_1.prettierConfig.init(true);
        Npmrc_1.npmrc.init(env_1.envConfig, true);
        // Meta files init
        dockerignore_1.dockerIgnore.init(env_1.envConfig, true);
        gitignore_1.gitIgnore.init(env_1.envConfig, true);
        console.log("");
        yield install_declared_deps_1.install_declared_deps(env_1.envConfig);
        yield install_dev_deps_1.install_dev_deps(env_1.envConfig);
        yield ordering_project_deps_1.ordering_project_deps(env_1.envConfig);
        yield sync_with_linked_modules_1.sync_with_linked_modules(env_1.envConfig);
        console.log(chalk_1.default.bold.blue(`[ START_NODE_SERVER ]`));
        console.log("");
        // tslint:disable-next-line: variable-name
        const tslint_command = `tslint -c tslint.json 'src/**/*.ts' 'bin/**/*.ts' --fix`;
        console.log(chalk_1.default.white(tslint_command));
        console.log("");
        child_process_1.execSync(tslint_command, {
            cwd: process.cwd(),
            encoding: "utf8",
            stdio: "inherit",
        });
        // tslint:disable-next-line: variable-name
        const ts_node_dev_command = `ts-node-dev --prefer-ts --type-check --respawn ./bin/www.ts`;
        console.log(chalk_1.default.white(ts_node_dev_command));
        console.log("");
        const result = spawn.sync("ts-node-dev", ["--prefer-ts", "--type-check", "--respawn", "./bin/www.ts"], {
            encoding: "utf8",
            stdio: "inherit",
        });
        if (result.signal) {
            if (result.signal === "SIGKILL") {
                console.log(chalk_1.default.red("The build failed because the process exited too early. " +
                    "This probably means the system ran out of memory or someone called `kill -9` on the process."));
                process.exit(1);
            }
            else if (result.signal === "SIGTERM") {
                console.log(chalk_1.default.bold.red("The build failed because the process exited too early. " +
                    "Someone might have called `kill` or `killall`, or the system could be shutting down."));
                process.exit(1);
            }
            process.exit(0);
        }
        console.log("");
    });
}
start_node_server();
