"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const fs = require("fs");
const path = require("path");
const BuildStatusConfig_1 = require("../config/buildStatus/BuildStatusConfig");
const RearguardConfig_1 = require("../config/rearguard/RearguardConfig");
const const_1 = require("../const");
const build_1 = require("./mono_repository/build");
// tslint:disable: variable-name
function build_intermediate_dependencies(envConfig, rearguardConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(chalk_1.default.bold.white(`[ BUILD_INTERMEDIATE_DEPENDENCIES ][ START ]`));
        console.log(chalk_1.default.bold.white(`[ FROM: ${rearguardConfig.pkg.name} ]`));
        console.log("");
        // ! CUR_DEPENDECIES_LIST
        const cur_dep_list = rearguardConfig.sync_project_deps;
        // ! REBUIL_LIST
        const target_modules = [];
        for (const cur_dep_name of cur_dep_list) {
            const cur_local_dep = envConfig.resolveLocalModule(cur_dep_name);
            const cur_global_dep = envConfig.resolveGlobalModule(cur_dep_name);
            if (fs.existsSync(cur_global_dep)) {
                // ! GET_CONFIG_FILEs
                const cur_rearguardConfig = new RearguardConfig_1.RearguardConfig(envConfig, path.resolve(cur_global_dep, "package.json"));
                const cur_buildStatusConfig = new BuildStatusConfig_1.BuildStatusConfig(path.resolve(cur_global_dep, const_1.NON_VERSIONABLE_CONFIG_FILE_NAME));
                // ! GET_BUILD_INFORMATION
                const cur_deps = cur_rearguardConfig.sync_project_deps;
                const cur_has_last_build_time = cur_buildStatusConfig.has_last_build_time;
                if (cur_has_last_build_time) {
                    const cur_build_time = cur_buildStatusConfig.last_build_time;
                    for (let k = cur_deps.length - 1; k >= 0; k--) {
                        const k_dep_name = cur_deps[k];
                        const k_global_dep = envConfig.resolveGlobalModule(k_dep_name);
                        if (fs.existsSync(k_global_dep)) {
                            // ? GET_TARGET_CONFIG
                            const k_buildStatusConfig = new BuildStatusConfig_1.BuildStatusConfig(path.resolve(k_global_dep, const_1.NON_VERSIONABLE_CONFIG_FILE_NAME));
                            // ? GET_TARGET_BUILD_INFORMATION
                            const k_has_last_build_time = k_buildStatusConfig.has_last_build_time;
                            if (k_has_last_build_time) {
                                const k_build_time = k_buildStatusConfig.last_build_time;
                                // * CUR_BUILD_TIME <= TARGET_BUILD_TIME
                                if (cur_build_time.isSameOrBefore(k_build_time)) {
                                    target_modules.push(cur_global_dep);
                                    break;
                                }
                            }
                            else if (!target_modules.includes(cur_global_dep)) {
                                target_modules.push(cur_global_dep);
                            }
                        }
                        else {
                            console.log(chalk_1.default.bold.grey(`[ GLOBAL_MODULE: ${k_dep_name} ][ NOT_FOUND ]`));
                        }
                    }
                }
                else if (!target_modules.includes(cur_global_dep)) {
                    target_modules.push(cur_global_dep);
                }
            }
            else if (fs.existsSync(cur_local_dep)) {
                console.log(chalk_1.default.grey(`[ MODULE: ${cur_dep_name} ][ LOCAL_INSTALED ]`));
            }
            else {
                console.log(chalk_1.default.grey(`[ MODULE: ${cur_dep_name} ][ NOT_FOUND ]`));
            }
        }
        if (target_modules.length > 0) {
            console.log(chalk_1.default.bold.yellowBright(`[ TARGET_MODULES ]`));
            console.log(chalk_1.default.bold.yellowBright(JSON.stringify(target_modules, null, 2)));
        }
        else {
            console.log(chalk_1.default.grey(`[ HAVE_NOT_TARGETS ]`));
        }
        for (const CWD of target_modules) {
            yield build_1.build(CWD, true);
        }
        console.log("");
        console.log(chalk_1.default.bold.white(`[ BUILD_INTERMEDIATE_DEPENDENCIES ][ END ]`));
        console.log("");
    });
}
exports.build_intermediate_dependencies = build_intermediate_dependencies;
// tslint:enable: variable-name
