"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const fs = require("fs");
const mkdirp = require("mkdirp");
const path = require("path");
const prettier_package_json = require("prettier-package-json");
const env_1 = require("../config/env");
const prettier_1 = require("../config/prettier");
const rearguard_1 = require("../config/rearguard");
const tslint_1 = require("../config/tslint");
const typescript_1 = require("../config/typescript");
const const_1 = require("../const");
const dockerignore_1 = require("../meta/dockerignore");
const editorConfig_1 = require("../meta/editorConfig");
const gitignore_1 = require("../meta/gitignore");
const postcssPlugins_1 = require("../meta/postcssPlugins");
const PrePublish_1 = require("../meta/PrePublish");
const Typings_1 = require("../meta/Typings");
const index_1 = require("./../meta/Npmrc/index");
const copy_bundles_1 = require("./project_deps/copy_bundles");
const delete_bundles_1 = require("./project_deps/delete_bundles");
const install_declared_deps_1 = require("./project_deps/install_declared_deps");
const install_dev_deps_1 = require("./project_deps/install_dev_deps");
const ordering_project_deps_1 = require("./project_deps/ordering_project_deps");
const set_list_of_modules_with_deferred_loading_1 = require("./project_deps/set_list_of_modules_with_deferred_loading");
const sync_with_linked_modules_1 = require("./project_deps/sync_with_linked_modules");
// tslint:disable:variable-name
function initProject() {
    return __awaiter(this, void 0, void 0, function* () {
        console.log(chalk_1.default.bold.magenta(`================INIT_PROJECT================`));
        console.log("");
        const pkg_path = path.resolve(process.cwd(), "package.json");
        if (!fs.existsSync(pkg_path)) {
            console.log(chalk_1.default.red(`[ INIT ][ ERROR ][ You haven't package.json, you should do npm init ]`));
            console.log("");
            process.exit(1);
        }
        const update_for_pkg = {};
        /**
         * ! INIT PROJECT STATUS
         */
        if (env_1.envConfig.isInit) {
            rearguard_1.rearguardConfig.has_dll = env_1.envConfig.has_dll;
            rearguard_1.rearguardConfig.has_browser_lib = env_1.envConfig.has_browser_lib;
            rearguard_1.rearguardConfig.has_node_lib = env_1.envConfig.has_node_lib;
            rearguard_1.rearguardConfig.load_on_demand = env_1.envConfig.load_on_demand;
            rearguard_1.rearguardConfig.is_application = env_1.envConfig.is_application;
            rearguard_1.rearguardConfig.is_back_end = env_1.envConfig.is_back_end;
        }
        if (!rearguard_1.rearguardConfig.is_back_end) {
            const files = [];
            if (rearguard_1.rearguardConfig.has_dll) {
                files.push(const_1.DLL_BUNDLE_DIR_NAME);
            }
            if (rearguard_1.rearguardConfig.has_browser_lib) {
                files.push(const_1.LIB_BUNDLE_DIR_NAME);
            }
            if (rearguard_1.rearguardConfig.has_node_lib || rearguard_1.rearguardConfig.has_browser_lib) {
                files.push(const_1.LIB_DIR_NAME);
            }
            console.log(chalk_1.default.green(`[ FILES ][ ${files.join(", ")} ]`));
            console.log("");
            update_for_pkg.files = files;
        }
        /**
         * ! INIT ENTRY FILES
         */
        const src = path.resolve(process.cwd(), "src");
        mkdirp.sync(src);
        if (rearguard_1.rearguardConfig.is_back_end) {
            const bin = path.resolve(process.cwd(), "bin");
            const www = path.resolve(bin, "www.ts");
            mkdirp.sync(bin);
            if (!fs.existsSync(www)) {
                fs.writeFileSync(www, `console.trace("Точка входа в приложение!");`);
                console.log(chalk_1.default.green(`[ BACK_END ][ ENTRY_POINT ][ INIT ][ ${www} ]`));
                console.log("");
            }
        }
        if (!rearguard_1.rearguardConfig.is_back_end) {
            if (rearguard_1.rearguardConfig.has_browser_lib || rearguard_1.rearguardConfig.is_application) {
                const entry = path.resolve(src, rearguard_1.rearguardConfig.entry);
                if (!fs.existsSync(entry)) {
                    fs.writeFileSync(entry, `// Точка входа в проект;\n`);
                    console.log(chalk_1.default.green(`[ ENTRY_FILE ][ INIT ][ ${entry} ]`));
                    console.log("");
                }
            }
            if (rearguard_1.rearguardConfig.has_dll) {
                const dll_entry = path.resolve(src, rearguard_1.rearguardConfig.dll_entry);
                if (!fs.existsSync(dll_entry)) {
                    fs.writeFileSync(dll_entry, `// В этом файле указываются импорты пакетов, которые необходимо вынести в dll;\n`);
                    console.log(chalk_1.default.green(`[ ENTRY_FILE ][ INIT ][ ${dll_entry} ]`));
                    console.log("");
                }
            }
            if (rearguard_1.rearguardConfig.has_browser_lib || rearguard_1.rearguardConfig.has_node_lib) {
                const lib_entry = path.resolve(src, rearguard_1.rearguardConfig.lib_entry);
                if (!fs.existsSync(lib_entry)) {
                    fs.writeFileSync(lib_entry, `// Точка экспорта API из библиотеки;\n`);
                    console.log(chalk_1.default.green(`[ ENTRY_FILE ][ INIT ][ ${lib_entry} ]`));
                    console.log("");
                }
            }
            if (rearguard_1.rearguardConfig.has_browser_lib || rearguard_1.rearguardConfig.has_node_lib) {
                const lib_entry = path.resolve(src, rearguard_1.rearguardConfig.lib_entry);
                const basename = path.basename(lib_entry, ".ts");
                update_for_pkg.main = `${const_1.LIB_DIR_NAME}/${basename}.js`;
                update_for_pkg.module = `${const_1.LIB_DIR_NAME}/${basename}.js`;
                update_for_pkg.types = `${const_1.LIB_DIR_NAME}/${basename}.d.ts`;
            }
        }
        /**
         * ! INIT SCRIPTS
         */
        if (!update_for_pkg.scripts) {
            const { scripts } = require(pkg_path);
            if (utils_1.isObject(scripts)) {
                update_for_pkg.scripts = Object.assign({}, scripts);
                delete update_for_pkg.scripts.build;
                delete update_for_pkg.scripts["build:release"];
                delete update_for_pkg.scripts["build:both"];
                delete update_for_pkg.scripts.start;
                delete update_for_pkg.scripts["start:release"];
                delete update_for_pkg.scripts.dll;
                delete update_for_pkg.scripts["dll:release"];
                delete update_for_pkg.scripts.check_deps_on_npm;
                delete update_for_pkg.scripts["check_deps_on_npm:install"];
                delete update_for_pkg.scripts.prepublishOnly;
            }
            else {
                update_for_pkg.scripts = {};
            }
        }
        if (!rearguard_1.rearguardConfig.is_back_end) {
            /**
             * ! BUILD_SCRIPTS
             */
            const args = [];
            if (rearguard_1.rearguardConfig.has_dll) {
                args.push("--dll");
            }
            if (rearguard_1.rearguardConfig.has_browser_lib) {
                args.push("--browser_lib");
            }
            if (rearguard_1.rearguardConfig.has_node_lib) {
                args.push("--node_lib");
            }
            update_for_pkg.scripts.build = "rearguard build " + args.join(" ");
            update_for_pkg.scripts["build:release"] = "rearguard build -r " + args.join(" ");
            update_for_pkg.scripts["build:both"] = "rearguard build --both " + args.join(" ");
            /**
             * ! START
             */
            if (rearguard_1.rearguardConfig.is_application) {
                update_for_pkg.scripts.start = "rearguard wds";
                update_for_pkg.scripts["start:release"] = "rearguard wds -r";
                if (rearguard_1.rearguardConfig.has_dll) {
                    if (!(rearguard_1.rearguardConfig.has_browser_lib || rearguard_1.rearguardConfig.has_node_lib)) {
                        update_for_pkg.scripts.build = update_for_pkg.scripts.build.replace("--dll", "");
                        update_for_pkg.scripts["build:release"] = update_for_pkg.scripts["build:release"].replace("--dll", "");
                        update_for_pkg.scripts["build:both"] = update_for_pkg.scripts["build:both"].replace("--dll", "");
                    }
                    update_for_pkg.scripts.dll = "rearguard build --dll";
                    update_for_pkg.scripts["dll:release"] = "rearguard build --dll -r";
                }
                if (!(rearguard_1.rearguardConfig.has_browser_lib || rearguard_1.rearguardConfig.has_node_lib)) {
                    update_for_pkg.scripts.build += "--application";
                    update_for_pkg.scripts["build:release"] += "--application";
                    update_for_pkg.scripts["build:both"] += "--application";
                }
            }
            else {
                delete update_for_pkg.scripts.start;
                delete update_for_pkg.scripts["start:release"];
                delete update_for_pkg.scripts.dll;
                delete update_for_pkg.scripts["dll:release"];
            }
            if (!rearguard_1.rearguardConfig.is_application && rearguard_1.rearguardConfig.has_browser_lib) {
                update_for_pkg.scripts.start = "rearguard wds";
                update_for_pkg.scripts["start:release"] = "rearguard wds -r";
            }
            update_for_pkg.scripts.check_deps_on_npm = "rearguard check_deps_on_npm";
            update_for_pkg.scripts["check_deps_on_npm:install"] = "rearguard check_deps_on_npm --install_deps";
            /**
             * ! PRE_PUBLISH_ONLY
             */
            if (!rearguard_1.rearguardConfig.has_dll && !rearguard_1.rearguardConfig.has_browser_lib && rearguard_1.rearguardConfig.has_node_lib) {
                update_for_pkg.scripts.prepublishOnly = "npm run build:release";
            }
            else {
                update_for_pkg.scripts.prepublishOnly = "npm run build:both";
            }
        }
        else {
            update_for_pkg.scripts.start = "rearguard start_node_server";
            update_for_pkg.scripts.build = "rearguard build_node_server";
        }
        // ! TEST SCRIPT
        update_for_pkg.scripts.test = "rearguard test";
        const cur_pkg = require(pkg_path);
        const pkg_text = prettier_package_json.format(Object.assign({}, cur_pkg, update_for_pkg));
        fs.writeFileSync(pkg_path, pkg_text.trim());
        // Config file
        typescript_1.typescriptConfig.init(true);
        tslint_1.tsLintConfig.init(true);
        prettier_1.prettierConfig.init(true);
        // Meta files init
        dockerignore_1.dockerIgnore.init(env_1.envConfig, true);
        gitignore_1.gitIgnore.init(env_1.envConfig, true);
        editorConfig_1.editorConfig.init(env_1.envConfig, true);
        index_1.npmrc.init(env_1.envConfig, true);
        if (!rearguard_1.rearguardConfig.is_back_end) {
            if (rearguard_1.rearguardConfig.is_application || rearguard_1.rearguardConfig.has_browser_lib) {
                PrePublish_1.prePublish.init(env_1.envConfig, true);
                Typings_1.typings.init(env_1.envConfig, true);
                postcssPlugins_1.postcssPlugins.init(env_1.envConfig);
            }
        }
        console.log("");
        yield install_declared_deps_1.install_declared_deps(env_1.envConfig);
        yield install_dev_deps_1.install_dev_deps(env_1.envConfig);
        yield ordering_project_deps_1.ordering_project_deps(env_1.envConfig);
        yield sync_with_linked_modules_1.sync_with_linked_modules(env_1.envConfig);
        if (!rearguard_1.rearguardConfig.is_back_end) {
            if (rearguard_1.rearguardConfig.is_application || rearguard_1.rearguardConfig.has_dll || rearguard_1.rearguardConfig.has_browser_lib) {
                yield delete_bundles_1.delete_bundles(env_1.envConfig, rearguard_1.rearguardConfig);
                yield copy_bundles_1.copy_bundles(env_1.envConfig);
            }
            set_list_of_modules_with_deferred_loading_1.set_list_of_modules_with_deferred_loading(env_1.envConfig, rearguard_1.rearguardConfig);
        }
        rearguard_1.rearguardConfig.order_config_fields();
        console.log(chalk_1.default.bold.magenta(`============================================`));
        console.log("");
    });
}
exports.initProject = initProject;
// tslint:enable:variable-name
