"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const spawn = require("cross-spawn");
const path = require("path");
const env_1 = require("../../config/env");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
function build(CWD, needUpdateBuildTime = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const rearguardConfig = new RearguardConfig_1.RearguardConfig(env_1.envConfig, path.resolve(CWD, "package.json"));
        console.log(chalk_1.default.bold.green(`[ ${rearguardConfig.pkg.name} ][ BUILD ]`));
        console.log("");
        let result;
        const srcipts = rearguardConfig.pkg.scripts;
        if (srcipts && Object.keys(srcipts).length > 0) {
            if (env_1.envConfig.isBuildBoth) {
                // tslint:disable-next-line:variable-name
                result = spawn.sync("rearguard", [
                    ...srcipts["build:both"].replace("rearguard ", "").split(" "),
                    ...(needUpdateBuildTime ? ["--needUpdateBuildTime"] : []),
                ], {
                    cwd: CWD,
                    encoding: "utf8",
                    stdio: "inherit",
                });
            }
            else {
                if (!env_1.envConfig.isDevelopment) {
                    result = spawn.sync("rearguard", [
                        ...srcipts["build:release"].replace("rearguard ", "").split(" "),
                        ...(needUpdateBuildTime ? ["--needUpdateBuildTime"] : []),
                    ], {
                        cwd: CWD,
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                }
                if (env_1.envConfig.isDevelopment) {
                    result = spawn.sync("rearguard", [
                        ...srcipts.build.replace("rearguard ", "").split(" "),
                        ...(needUpdateBuildTime ? ["--needUpdateBuildTime"] : []),
                    ], {
                        cwd: CWD,
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                }
            }
            // ! Обработка сигнала.
            if (result.signal) {
                if (result.signal === "SIGKILL") {
                    console.log(chalk_1.default.red("The build failed because the process exited too early. " +
                        "This probably means the system ran out of memory or someone called `kill -9` on the process."));
                    process.exit(1);
                }
                else if (result.signal === "SIGTERM") {
                    console.log(chalk_1.default.bold.red("The build failed because the process exited too early. " +
                        "Someone might have called `kill` or `killall`, or the system could be shutting down."));
                    process.exit(1);
                }
                process.exit(0);
            }
        }
        console.log(chalk_1.default.green(`[ ${rearguardConfig.pkg.name} ][ BUILD ][ END ]`));
        console.log("");
    });
}
exports.build = build;
