"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const spawn = require("cross-spawn");
const ora_1 = require("ora");
const path = require("path");
const semver = require("semver");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
const check_npm_1 = require("../check_npm");
// tslint:disable:variable-name
function publish(envConfig, CWD) {
    return __awaiter(this, void 0, void 0, function* () {
        const rearguardConfig = new RearguardConfig_1.RearguardConfig(envConfig, path.resolve(CWD, "package.json"));
        const npmIsAvailable = yield check_npm_1.check_npm();
        if (npmIsAvailable && !rearguardConfig.publish_in_git) {
            // * INIT VALUES
            const cur_name = rearguardConfig.pkg.name;
            const cur_version = rearguardConfig.pkg.version;
            let pub_version = "1.0.0";
            let was_published = false;
            console.log(chalk_1.default.bold.green(`[ ${cur_name} ][ PUBLISH ]`));
            console.log("");
            // * SEARCH IN NPM REGISTRY
            const spinner = ora_1.default(`npm search --json ${cur_name}`).start();
            const search_result = child_process_1.execSync(`npm search --json ${cur_name}`, { encoding: "utf8" });
            try {
                // * PARSING_RESULT
                const result = JSON.parse(search_result);
                spinner.succeed();
                console.log("");
                // * FIND_CUR_MODULE
                if (Array.isArray(result) && result.length > 0) {
                    for (const item of result) {
                        if (item.name === cur_name) {
                            was_published = true;
                            pub_version = item.version;
                            break;
                        }
                    }
                }
                if (was_published) {
                    const { is_mono_publish_patch, is_mono_publish_minor, is_mono_publish_major } = envConfig;
                    if (!(is_mono_publish_patch || is_mono_publish_minor || is_mono_publish_major)) {
                        if (cur_version === pub_version) {
                            return;
                        }
                        if (semver.lt(cur_version, pub_version)) {
                            rearguardConfig.pkg = { version: pub_version };
                            return;
                        }
                    }
                    if ((is_mono_publish_patch || is_mono_publish_minor || is_mono_publish_major) &&
                        (cur_version === pub_version || semver.lt(cur_version, pub_version))) {
                        let release = "patch";
                        if (envConfig.is_mono_publish_minor) {
                            release = "minor";
                        }
                        if (envConfig.is_mono_publish_major) {
                            release = "major";
                        }
                        const target_version = semver.inc(pub_version, release);
                        if (utils_1.isString(target_version) && semver.valid(target_version)) {
                            rearguardConfig.pkg = { version: target_version };
                        }
                        else {
                            throw new Error(`[ MONOREPO ][ PUBLISH ][ target version is not valid ]`);
                        }
                    }
                }
                if (semver.valid(rearguardConfig.pkg.version)) {
                    const publish_result = spawn.sync("npm", ["publish"], {
                        cwd: CWD,
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                    if (publish_result.signal) {
                        if (publish_result.signal === "SIGKILL") {
                            console.log(chalk_1.default.red("The build failed because the process exited too early. " +
                                "This probably means the system ran out of memory or someone called `kill -9` on the process."));
                            process.exit(1);
                        }
                        else if (publish_result.signal === "SIGTERM") {
                            console.log(chalk_1.default.bold.red("The build failed because the process exited too early. " +
                                "Someone might have called `kill` or `killall`, or the system could be shutting down."));
                            process.exit(1);
                        }
                        process.exit(0);
                    }
                }
                else {
                    throw new Error(`[ MONOREPO ][ PUBLISH ][ current version is not valid ]`);
                }
            }
            catch (error) {
                console.error(error);
                spinner.fail();
                console.log("");
            }
            console.log(chalk_1.default.green(`[ ${cur_name} ][ PUBLISH ][ END ]`));
            console.log("");
        }
        else {
            console.log(chalk_1.default.yellowBright(`[ PUBLISH ][ SKIPED ]`));
            console.log("");
        }
    });
}
exports.publish = publish;
// tslint:enable:variable-name
