"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const child_process_1 = require("child_process");
const path = require("path");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
const check_npm_1 = require("../check_npm");
function install_dev_deps(envConfig, CWD = process.cwd()) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { pkg } = new RearguardConfig_1.RearguardConfig(envConfig, path.resolve(CWD, "package.json"));
            let npmIsAvailable = false;
            console.log(chalk_1.default.bold.blue(`[ INSTALL_DEV_DEPS ][ START ]`));
            console.log("");
            /////////////////////
            //
            // * START OF PROCEDURE
            //
            /////////////////////
            // ! Install dev dependencies
            const devDeps = Object.keys(pkg.devDependencies || {});
            const installDevDepsList = [];
            for (const depName of [
                "typescript",
                "tslint",
                "ts-node-dev",
                "ts-node",
                "prettier",
                "husky",
                "mocha",
                "chai",
                "@types/node",
                "@types/mocha",
                "@types/chai",
            ]) {
                if (!devDeps.includes(depName)) {
                    installDevDepsList.push(depName);
                }
            }
            const deps = Object.keys(pkg.dependencies || {});
            const installDepsList = [];
            for (const depName of ["tslib"]) {
                if (!deps.includes(depName)) {
                    installDepsList.push(depName);
                }
            }
            if (installDevDepsList.length > 0) {
                if (!npmIsAvailable) {
                    npmIsAvailable = yield check_npm_1.check_npm();
                }
                if (npmIsAvailable) {
                    // tslint:disable-next-line: variable-name
                    const command = `npm install -D -E ${installDevDepsList.join(" ")}`;
                    console.log(chalk_1.default.white(command));
                    console.log(chalk_1.default.white("npm install"));
                    console.log("");
                    child_process_1.execSync(command, {
                        cwd: process.cwd(),
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                }
            }
            else {
                console.log(chalk_1.default.white(`Dev dependencies alredy installed`));
                console.log("");
            }
            // ! Install dependencies
            if (installDepsList.length > 0) {
                if (!npmIsAvailable) {
                    npmIsAvailable = yield check_npm_1.check_npm();
                }
                if (npmIsAvailable) {
                    const command = `npm install -S -E ${installDepsList.join(" ")}`;
                    console.log(chalk_1.default.white(command));
                    console.log(chalk_1.default.white("npm install"));
                    console.log("");
                    child_process_1.execSync(command, {
                        cwd: process.cwd(),
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                }
            }
            else {
                console.log(chalk_1.default.white(`Dependencies alredy installed`));
                console.log("");
            }
            if (installDevDepsList.length > 0 || installDepsList.length > 0) {
                if (!npmIsAvailable) {
                    npmIsAvailable = yield check_npm_1.check_npm();
                }
                if (npmIsAvailable) {
                    child_process_1.execSync("npm install", {
                        cwd: process.cwd(),
                        encoding: "utf8",
                        stdio: "inherit",
                    });
                    console.log("");
                    return true;
                }
            }
            /////////////////////
            //
            // * END OF PROCEDURE
            //
            /////////////////////
            console.log(chalk_1.default.bold.blue(`[ INSTALL_DEV_DEPS ][ END ]`));
            console.log("");
        }
        catch (error) {
            console.trace(chalk_1.default.red(`[ ERROR_MESSAGE: ${utils_1.getErrorMessage(error)} ]`));
            process.exit(1);
        }
        return false;
    });
}
exports.install_dev_deps = install_dev_deps;
