"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const copy = require("copy");
const del_1 = require("del");
const fs_1 = require("fs");
const mkdirp = require("mkdirp");
const path = require("path");
const semver = require("semver");
const util_1 = require("util");
const rearguard_1 = require("../../config/rearguard");
const RearguardConfig_1 = require("../../config/rearguard/RearguardConfig");
function sync_with_linked_modules(envConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { sync_project_deps } = rearguard_1.rearguardConfig;
            if (sync_project_deps.length > 0) {
                console.log(chalk_1.default.bold.blue(`==========SYNC_WITH_LINKED_MODULES==========`));
                console.log("");
                /////////////////////
                //
                // * START OF PROCEDURE
                //
                /////////////////////
                console.log(chalk_1.default.bold.white(`[ LIST ][ ${sync_project_deps.join(", ")} ]`));
                console.log("");
                const linked_modules = [];
                const copyTasks = [];
                /////////////////////
                //
                // * Проверка на существование модулей;
                // * Составление списка модулей;
                //
                /////////////////////
                for (const name of sync_project_deps) {
                    const global_path = envConfig.resolveGlobalModule(name);
                    const local_path = envConfig.resolveLocalModule(name);
                    if (fs_1.existsSync(global_path)) {
                        console.log(chalk_1.default.white(`[ USED ][ GLOBAL_MODULE: ${global_path} ]`));
                        linked_modules.push({ name, path: global_path });
                    }
                    else if (fs_1.existsSync(local_path)) {
                        // Внимание, modules_path - в этот список необходимо добавлять только глобально слинкованные модули.
                        // Так как существует ситуация когда зависимости устанавдиваются локально, и не разрабатываются.
                        console.log(chalk_1.default.bold.yellow(`[ USED ][ LOCAL_MODULE: ${local_path} ]`));
                    }
                    else {
                        console.log(chalk_1.default.red(`[ ERROR ]` +
                            `[ You haven't link in global node_modules ${global_path} or local node_modules ${local_path} ]`));
                        console.log(chalk_1.default.red(`[ ERROR ]` + `[ You need go to the project and do [ npm link || npm install ] needs module ]`));
                        process.exit(1);
                    }
                }
                console.log("");
                // END
                if (linked_modules.length > 0) {
                    /////////////////////
                    //
                    // * Составляю задач для копирования слинкованных модулей;
                    //
                    /////////////////////
                    for (const module of linked_modules) {
                        const pkg_path = path.resolve(module.path, "package.json");
                        const config = new RearguardConfig_1.RearguardConfig(envConfig, pkg_path);
                        if (fs_1.existsSync(pkg_path)) {
                            const files = [pkg_path];
                            if (util_1.isArray(config.pkg.files)) {
                                for (const file_name of config.pkg.files) {
                                    const file_path = path.resolve(module.path, file_name);
                                    if (fs_1.existsSync(file_path)) {
                                        files.push(`${file_path}/**`);
                                    }
                                    else {
                                        console.log(chalk_1.default.red(`[ COPY_TASK ][ ERROR ][ module haven't file: ${file_path} ]`));
                                        console.log("");
                                        process.exit(1);
                                    }
                                }
                            }
                            else {
                                console.log(chalk_1.default.red(`[ COPY_TASK ][ ERROR ][ module must defined 'files: []' ]`));
                                console.log("");
                                process.exit(1);
                            }
                            copyTasks.push({ version: config.pkg.version, name: module.name, files });
                        }
                        else {
                            console.log(chalk_1.default.red(`[ COPY_TASK ][ ERROR ][ not found: ${pkg_path} ]`));
                            console.log("");
                            process.exit(1);
                        }
                    }
                    // END
                    /////////////////////
                    //
                    // * Фильтрация модулей, на те для которых есть слинованные замены и на те для которых нет.
                    // * Удаление модулей для которых слинкованная замена.
                    // * Восстановление директорий модулей для последующего наполнения.
                    //
                    /////////////////////
                    const modules_for_remove = sync_project_deps
                        .filter((name) => {
                        for (const module of linked_modules) {
                            if (module.name === name) {
                                return true;
                            }
                        }
                        return false;
                    })
                        .map((name) => envConfig.resolveLocalModule(name));
                    for (const module_path of modules_for_remove) {
                        if (fs_1.existsSync(module_path)) {
                            const paths = yield del_1.default(module_path);
                            for (const item of paths) {
                                console.log(chalk_1.default.gray(`[ MODULE ][ REMOVE ][ ${path.relative(process.cwd(), item)} ]`));
                            }
                        }
                        yield new Promise((resolve, reject) => {
                            mkdirp(module_path, (error) => {
                                if (error) {
                                    console.error(error);
                                    reject();
                                }
                                else {
                                    console.log(chalk_1.default.green(`[ MODULE ][ CREATED ][ DIR ][ ${path.relative(process.cwd(), module_path)} ]`));
                                    resolve();
                                }
                            });
                        });
                    }
                    console.log("");
                    // END
                    /////////////////////
                    //
                    // * Выполнение задачь по копированияю.
                    //
                    /////////////////////
                    const LOCAL_PKG = Object.assign({}, rearguard_1.rearguardConfig.pkg);
                    for (const task of copyTasks) {
                        yield new Promise((resolve, reject) => {
                            copy(task.files, envConfig.resolveLocalModule(task.name), (error, items) => {
                                if (!error) {
                                    if (envConfig.isDebug) {
                                        for (const item of items) {
                                            console.log(chalk_1.default.cyan(`[ MODULE ][ COPY ][ ${task.name} ]`), chalk_1.default.cyan(item.path));
                                        }
                                    }
                                    else {
                                        console.log(chalk_1.default.cyan(`[ MODULE ][ COPY ][ ${task.name} ]`), chalk_1.default.cyan(`[ ${items.length} FILES ]`));
                                    }
                                    resolve();
                                }
                                else {
                                    reject(error);
                                }
                            });
                        });
                        for (const dep_type of ["dependencies", "devDependencies", "peerDependencies"]) {
                            if (!utils_1.isUndefined(LOCAL_PKG[dep_type]) && utils_1.isString(LOCAL_PKG[dep_type][task.name])) {
                                if (semver.valid(LOCAL_PKG[dep_type][task.name])) {
                                    LOCAL_PKG[dep_type][task.name] = task.version;
                                }
                            }
                        }
                    }
                    // * Обновляем локальный package.json
                    rearguard_1.rearguardConfig.pkg = LOCAL_PKG;
                    console.log("");
                    console.log(chalk_1.default.bold.white(`[ UPDATED VERSIONS OF LINKED MODULES IN LOCAL PACKAGE.JSON ]`));
                    console.log("");
                    // END
                }
                else {
                    console.log(chalk_1.default.bold.gray("[ LINKED_MODULES_NOT_FOUNT ]"));
                    console.log(chalk_1.default.bold.gray("[ USED ONLY LOCAL INSTALLED MODULES ]"));
                    console.log("");
                }
                /////////////////////
                //
                // * END OF PROCEDURE
                //
                /////////////////////
                console.log(chalk_1.default.bold.blue(`============================================`));
                console.log("");
            }
        }
        catch (error) {
            console.trace(chalk_1.default.red(`[ ERROR_MESSAGE: ${utils_1.getErrorMessage(error)} ]`));
            process.exit(1);
        }
    });
}
exports.sync_with_linked_modules = sync_with_linked_modules;
// tslint:enable:variable-name
