"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const fs = require("fs");
const lodash_1 = require("lodash");
const mkdirp = require("mkdirp");
const path = require("path");
const env_1 = require("./env");
// tslint:disable:variable-name
class ConfigFile {
    constructor(file_name) {
        if (utils_1.isString(file_name)) {
            this.config_path = path.resolve(process.cwd(), file_name);
        }
        else {
            throw new Error(`[ ${this.constructor.name} ][ file_name must be a string ]`);
        }
    }
    get config() {
        if (fs.existsSync(this.config_path)) {
            try {
                return JSON.parse(fs.readFileSync(this.config_path).toString());
            }
            catch (error) {
                console.error(error);
                return this.init();
            }
        }
        else {
            return this.init();
        }
    }
    init(force = false) {
        if (fs.existsSync(this.config_path)) {
            try {
                if (!env_1.envConfig.force && !force) {
                    const config_from_file = fs.readFileSync(this.config_path);
                    const source_config = JSON.parse(config_from_file.toString());
                    if (utils_1.isObject(source_config)) {
                        const config = lodash_1.defaultsDeep(source_config, this.default_config);
                        fs.writeFileSync(this.config_path, JSON.stringify(config, null, 2).trim());
                        return config;
                    }
                    fs.writeFileSync(this.config_path, JSON.stringify(this.default_config, null, 2).trim());
                    return this.default_config;
                }
                else {
                    fs.writeFileSync(this.config_path, JSON.stringify(this.default_config, null, 2).trim());
                    console.log(chalk_1.default.yellow(`[ CONFIG_FILE ][ FORCE_INIT ][ ${this.config_path} ]`));
                    return this.default_config;
                }
            }
            catch (error) {
                console.error("[ CONFIG_FILE ][ INIT ][ ERROR ]");
                console.error(error);
                fs.unlinkSync(this.config_path);
                console.log(chalk_1.default.gray(`[ CONFIG_FILE ][ DELETE ] ${this.config_path}`));
                return this.init();
            }
        }
        else {
            mkdirp.sync(path.dirname(this.config_path));
            fs.writeFileSync(this.config_path, JSON.stringify(this.default_config, null, 2).trim());
            console.log(chalk_1.default.greenBright(`[ CONFIG_FILE ][ CREATE ] ${this.config_path}`));
            return this.default_config;
        }
    }
    get default_config() {
        return {};
    }
}
exports.ConfigFile = ConfigFile;
// tslint:enable:variable-name
