"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const fs = require("fs");
const path = require("path");
const prettier_package_json = require("prettier-package-json");
// tslint:disable:variable-name
const field_order = [
    "context",
    "entry",
    "dll_entry",
    "lib_entry",
    "modules",
    "output",
    "post_css_plugins_path",
    "sync_project_deps",
    "is_application",
    "is_back_end",
    "has_dll",
    "has_browser_lib",
    "load_on_demand",
    "has_node_lib",
    "publish_in_git",
    "docker_org_name",
];
class VersionableConfig {
    constructor(config_path) {
        this.config_path = path.resolve(process.cwd(), "package.json");
        if (utils_1.isString(config_path)) {
            this.config_path = config_path;
        }
    }
    get config() {
        return this.pkg.rearguard;
    }
    set config(fields) {
        const unsorted_rearguard = Object.assign({}, this.pkg.rearguard, fields);
        const sorted_rearguard = {};
        for (const key of field_order) {
            if (unsorted_rearguard.hasOwnProperty(key)) {
                sorted_rearguard[key] = unsorted_rearguard[key];
            }
        }
        const pkg = JSON.parse(prettier_package_json.format(this.pkg).trim());
        pkg.rearguard = sorted_rearguard;
        fs.writeFileSync(this.config_path, JSON.stringify(pkg, null, 2).trim(), { encoding: "utf-8" });
    }
    get pkg() {
        if (fs.existsSync(this.config_path)) {
            try {
                const pkg = JSON.parse(fs.readFileSync(this.config_path).toString());
                if (!utils_1.isObject(pkg.rearguard)) {
                    this.pkg = { rearguard: {} };
                    console.log(chalk_1.default.greenBright(`[ VERSIONABLE-CONFIG ][ CREATED ] FIELD 'rearguard' IN: ${this.config_path};`));
                    console.log("");
                    return this.pkg;
                }
                return pkg;
            }
            catch (error) {
                console.error(error);
                return {};
            }
        }
        else {
            console.trace(chalk_1.default.bold.red(`[ ${this.constructor.name} ][ ERROR ][ You haven't package.json here: ${this.config_path} ]`));
            process.exit(1);
            return {};
        }
    }
    set pkg(fields) {
        if (fs.existsSync(this.config_path)) {
            try {
                const pkg = JSON.parse(fs.readFileSync(this.config_path).toString());
                fs.writeFileSync(this.config_path, prettier_package_json.format(Object.assign({}, pkg, fields)).trim());
            }
            catch (error) {
                console.error(error);
                process.exit(1);
            }
        }
        else {
            console.trace(chalk_1.default.bold.red(`[ ${this.constructor.name} ][ ERROR ][ You haven't package.json here: ${this.config_path} ]`));
            process.exit(1);
        }
    }
    order_config_fields() {
        this.config = this.config;
    }
}
exports.VersionableConfig = VersionableConfig;
// tslint:enable:variable-name
