"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const moment = require("moment");
const NonVersionableConfig_1 = require("../NonVersionableConfig");
// tslint:disable: variable-name
class BuildStatusConfig extends NonVersionableConfig_1.NonVersionableConfig {
    // ! BUILD STATUS
    // ! Текущее состояние сборки.
    get status() {
        const { status } = this.config;
        if (status) {
            return status;
        }
        this.config = { status: "init" };
        return this.status;
    }
    start() {
        this.config = { status: "in_progress" };
    }
    end() {
        this.config = { status: "done" };
    }
    // ! BUILD_HASH
    // ! Результат хеш функции от результата сборки.
    get hash_dev() {
        const { hash_dev } = this.config;
        if (utils_1.isString(hash_dev)) {
            return hash_dev;
        }
    }
    set hash_dev(hash_dev) {
        if (utils_1.isString(hash_dev)) {
            this.config = { hash_dev };
        }
    }
    get hash_prod() {
        const { hash_prod } = this.config;
        if (utils_1.isString(hash_prod)) {
            return hash_prod;
        }
    }
    set hash_prod(hash_prod) {
        if (utils_1.isString(hash_prod)) {
            this.config = { hash_prod };
        }
    }
    // ! LAST_BUILD_TIME
    // ! Время последней сборки.
    get last_build_time() {
        const { last_build_time } = this.config;
        if (utils_1.isDate(last_build_time)) {
            return moment(new Date(last_build_time)).utc();
        }
        console.log(chalk_1.default.bold.yellow(`[ BUILD_STATUS_CONFIG ][ WARNING ][ last_build_time ][ must be a Date ]`));
        const def_last_build_time = moment().utc();
        this.config = { last_build_time: def_last_build_time.toDate() };
        console.log(chalk_1.default.bold.green(`[ BUILD_STATUS_CONFIG ][ INIT ][ last_build_time ]` +
            `[ assign to ${def_last_build_time.format("YYYY-MM-DD HH:mm ZZ")} ]`));
        console.log("");
        return this.last_build_time;
    }
    set last_build_time(last_build_time) {
        this.config = { last_build_time: last_build_time.utc().toISOString() };
    }
    get has_last_build_time() {
        const { last_build_time } = this.config;
        return !!utils_1.isDate(last_build_time);
    }
}
exports.BuildStatusConfig = BuildStatusConfig;
// tslint:enable: variable-name
