"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@borodindmitriy/utils");
const chalk_1 = require("chalk");
const lodash_1 = require("lodash");
const path = require("path");
const const_1 = require("../../const");
const VersionableConfig_1 = require("../VersionableConfig");
// tslint:disable:variable-name
class RearguardConfig extends VersionableConfig_1.VersionableConfig {
    constructor(envConfig, config_path) {
        super(config_path);
        this.envConfig = envConfig;
    }
    get context() {
        const { context } = this.config;
        if (utils_1.isString(context) && context.length > 0) {
            return context;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ context ][ must be a non empty string ]`));
        this.config = { context: "src" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ context ][ assign to 'src' ]`));
        console.log("");
        return this.context;
    }
    get docker_org_name() {
        const { docker_org_name } = this.config;
        if (utils_1.isString(docker_org_name) && docker_org_name.length > 0) {
            return docker_org_name;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ docker_org_name ][ must be a non empty string ]`));
        this.config = { docker_org_name: "docker_org_name" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ docker_org_name ][ assign to 'docker_org_name' ]`));
        console.log("");
        return this.docker_org_name;
    }
    get entry() {
        const { entry } = this.config;
        if (utils_1.isString(entry) && entry.length > 0) {
            return entry;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ entry ][ must be a non empty string ]`));
        this.config = { entry: "index.tsx" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ entry ][ assign to 'index.tsx' ]`));
        console.log("");
        return this.entry;
    }
    get dll_entry() {
        const { dll_entry } = this.config;
        if (utils_1.isString(dll_entry) && dll_entry.length > 0) {
            return dll_entry;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ dll_entry ][ must be a non empty string ]`));
        this.config = { dll_entry: "vendors.ts" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ dll_entry ][ assign to 'vendors.ts' ]`));
        console.log("");
        return this.dll_entry;
    }
    get lib_entry() {
        const { lib_entry } = this.config;
        if (utils_1.isString(lib_entry) && lib_entry.length > 0) {
            return lib_entry;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ lib_entry ][ must be a non empty string ]`));
        this.config = { lib_entry: "lib_exports.ts" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ lib_entry ][ assign to 'lib_exports.ts' ]`));
        console.log("");
        return this.lib_entry;
    }
    get modules() {
        const { modules: a_modules } = this.config;
        const modules = [];
        let has_error = false;
        if (utils_1.isArray(a_modules) && a_modules.length > 0) {
            for (const i_m of a_modules) {
                if (utils_1.isString(i_m)) {
                    modules.push(i_m);
                }
                else {
                    console.log(chalk_1.default.bold.red(`[ RERGUARD_CONFIG ][ ERROR ][ module: ${i_m} must be a string ]`));
                    has_error = true;
                }
            }
            if (!has_error && modules.length > 0) {
                console.log("");
                return [
                    ...modules.map((dir_name) => path.resolve(process.cwd(), dir_name)),
                    this.envConfig.resolveLocalModule(""),
                    this.envConfig.resolveDevModule(""),
                ];
            }
        }
        if (!has_error && modules.length === 0) {
            console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ modules ][ must be not empty array of string ]`));
        }
        if (modules.length === 0) {
            modules.push("src");
        }
        this.config = { modules };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ modules ][ assign to [ ${modules.join(", ")} ] ]`));
        console.log("");
        return this.modules;
    }
    get output() {
        const { output } = this.config;
        if (output &&
            utils_1.isString(output.path) &&
            output.path.length > 0 &&
            utils_1.isString(output.publicPath) &&
            output.publicPath.length) {
            return {
                path: path.resolve(process.cwd(), output.path),
                publicPath: output.publicPath,
            };
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ output ][ must include { path: string, publicPath: string } ]`));
        this.config = {
            output: {
                path: "dist",
                publicPath: "/",
            },
        };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ output ][ output assign to '{ path: "dist", publicPath: "/" }' ]`));
        console.log("");
        return this.output;
    }
    get bundle_public_path() {
        return path.normalize(`${this.output.publicPath}/${lodash_1.snakeCase(this.pkg.name)}/${const_1.BUNDLE_SUB_DIR()}/`);
    }
    get post_css_plugins_path() {
        const { post_css_plugins_path } = this.config;
        if (utils_1.isString(post_css_plugins_path) && post_css_plugins_path.length > 0) {
            return post_css_plugins_path;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ post_css_plugins_path ][ must be a non empty string ]`));
        this.config = { post_css_plugins_path: "post_css_plugins.js" };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ post_css_plugins_path ][ assign to 'post_css_plugins.js' ]`));
        console.log("");
        return this.post_css_plugins_path;
    }
    get sync_project_deps() {
        const { sync_project_deps: a_sync_project_deps } = this.config;
        const sync_project_deps = [];
        let has_error = false;
        if (utils_1.isArray(a_sync_project_deps)) {
            for (const i_s of a_sync_project_deps) {
                if (utils_1.isString(i_s)) {
                    sync_project_deps.push(i_s);
                }
                else {
                    console.log(chalk_1.default.bold.red(`[ RERGUARD_CONFIG ][ ERROR ][ sync_npm_dep: ${i_s} must be a string ]`));
                    has_error = true;
                }
            }
            if (!has_error) {
                return sync_project_deps;
            }
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ sync_project_deps ][ must be not empty array of string ]`));
        this.config = { sync_project_deps };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ sync_project_deps ][ assign to [ ${sync_project_deps.join(", ")} ] ]`));
        console.log("");
        return this.sync_project_deps;
    }
    set sync_project_deps(sync_project_deps) {
        this.config = { sync_project_deps };
    }
    /**
     * Среда может содержать три флага:
     * has_dll: boolean; - говорит, о том, что в директории dll_bundle/%(package.json).name% собран бандл и manifest.json;
     * has_node_lib: boolean; - говорит, о том, что в директории lib находятся .js, .d.ts файлы;
     * has_browser_lib: boolean; - говорит, о том, что в директории lib_bundle/%(package.json).name% собран бандл и
     * manifest.json;
     *
     * Эти флаги используются только для копирования собранных файлов из директорий dll_bundle, lib_bundle, lib;
     */
    // HAS_DLL
    // Говорит, о том, что в директории dll_bundle/%(package.json).name% собран бандл и manifest.json;
    get has_dll() {
        const { has_dll } = this.config;
        if (utils_1.isBoolean(has_dll)) {
            return has_dll;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ has_dll ][ must be a boolean ]`));
        this.config = { has_dll: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ has_dll ][ assign to 'false' ]`));
        console.log("");
        return this.has_dll;
    }
    set has_dll(has_dll) {
        this.config = { has_dll };
    }
    // HAS_NODE_LIB
    // Говорит, о том, что в директории lib находятся .js, .d.ts файлы;
    get has_node_lib() {
        const { has_node_lib } = this.config;
        if (utils_1.isBoolean(has_node_lib)) {
            return has_node_lib;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ has_node_lib ][ must be a boolean ]`));
        this.config = { has_node_lib: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ has_node_lib ][ assign to 'false' ]`));
        console.log("");
        return this.has_node_lib;
    }
    set has_node_lib(has_node_lib) {
        this.config = { has_node_lib };
    }
    // has_browser_lib
    // Говорит, о том, что в директории lib_bundle/%(package.json).name% собран бандл и assets.json;
    get has_browser_lib() {
        const { has_browser_lib } = this.config;
        if (utils_1.isBoolean(has_browser_lib)) {
            return has_browser_lib;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ has_browser_lib ][ must be a boolean ]`));
        this.config = { has_browser_lib: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ has_browser_lib ][ assign to 'false' ]`));
        console.log("");
        return this.has_browser_lib;
    }
    set has_browser_lib(has_browser_lib) {
        this.config = { has_browser_lib };
    }
    // IS_APPLICATION
    // Говорит, о том, что проект можно собарть как файлы без экспортов, для заливки на сервер;
    get is_application() {
        const { is_application } = this.config;
        if (utils_1.isBoolean(is_application)) {
            return is_application;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ is_application ][ must be a boolean ]`));
        this.config = { is_application: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ is_application ][ assign to 'false' ]`));
        console.log("");
        return this.is_application;
    }
    set is_application(is_application) {
        this.config = { is_application };
    }
    // IS_BACK_END
    // Говорит о том что это back-end;
    get is_back_end() {
        const { is_back_end } = this.config;
        if (utils_1.isBoolean(is_back_end)) {
            return is_back_end;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ is_back_end ][ must be a boolean ]`));
        this.config = { is_back_end: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ is_back_end ][ assign to 'false' ]`));
        console.log("");
        return this.is_back_end;
    }
    set is_back_end(is_back_end) {
        this.config = { is_back_end };
    }
    // LOAD_ON_DEMAND
    // Говорит, о том, что библиотека не будет автоматически добавлена в index.html, а будет
    // добавлена информация о том, где она расположена на сервере, для загрузки во времени выполнения.
    get load_on_demand() {
        const { load_on_demand } = this.config;
        if (utils_1.isBoolean(load_on_demand)) {
            return load_on_demand;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ load_on_demand ][ must be a boolean ]`));
        this.config = { load_on_demand: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ load_on_demand ][ assign to 'false' ]`));
        console.log("");
        return this.load_on_demand;
    }
    set load_on_demand(load_on_demand) {
        this.config = { load_on_demand };
    }
    // PUBLISH_IN_GIT
    // Говорит о том, что необходимо оставить под версионированием директории указанные в (package.json).files;
    get publish_in_git() {
        const { publish_in_git } = this.config;
        if (utils_1.isBoolean(publish_in_git)) {
            return publish_in_git;
        }
        console.log(chalk_1.default.bold.yellow(`[ RERGUARD_CONFIG ][ WARNING ][ publish_in_git ][ must be a boolean ]`));
        this.config = { publish_in_git: false };
        console.log(chalk_1.default.bold.green(`[ RERGUARD_CONFIG ][ INIT ][ publish_in_git ][ assign to 'false' ]`));
        console.log("");
        return this.publish_in_git;
    }
}
exports.RearguardConfig = RearguardConfig;
// tslint:enable:variable-name
