"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const env_1 = require("../../config/env");
const helpers_1 = require("../../helpers");
exports.default = () => {
    const { isDebug, isDevelopment } = env_1.envConfig;
    const style = [{ loader: "isomorphic-style-loader" }];
    const css = [
        {
            loader: "css-loader",
            options: {
                // CSS Modules https://github.com/css-modules/css-modules
                sourceMap: isDebug,
            },
        },
    ];
    const cssModules = [
        {
            loader: "css-loader",
            options: {
                importLoaders: 1,
                localIdentName: isDevelopment ? "[path][local][hash:base64:4]" : "[hash:base64:32]",
                // CSS Modules https://github.com/css-modules/css-modules
                modules: true,
                sourceMap: isDebug,
            },
        },
    ];
    const postCss = [
        {
            loader: "postcss-loader",
            options: {
                ident: "postcss",
                plugins: require(path.resolve(__dirname, "postcss.config.js")),
            },
        },
    ];
    return [
        {
            exclude(modulePath) {
                return /node_modules/.test(modulePath);
            },
            include: helpers_1.get_context(),
            test: /\.css/,
            use: [...style, ...cssModules, ...postCss],
        },
        {
            exclude: helpers_1.get_context(),
            test: /\.css/,
            use: [...style, ...css],
        },
    ];
};
