"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const path = require("path");
const webpack = require("webpack");
const get_bundles_info_1 = require("../components/project_deps/get_bundles_info");
const helpers_1 = require("../helpers");
const css_loders_1 = require("./components/css.loders");
const js_plugins_1 = require("./components/js.plugins");
// tslint:disable:variable-name object-literal-sort-keys
function general_WP_config(envConfig, rearguardConfig, entry, output, rules, plugins, a_externals) {
    const { modules } = rearguardConfig;
    const { isDevelopment, isDebug } = envConfig;
    const info = get_bundles_info_1.get_bundles_info(envConfig, rearguardConfig);
    const lib_externals = {};
    for (const { has_browser_lib, bundle_name, pkg_name } of info) {
        if (has_browser_lib) {
            lib_externals[pkg_name] = {
                var: helpers_1.lib_entry_name(bundle_name),
            };
        }
    }
    const externals = Object.assign({}, lib_externals, a_externals);
    console.log(chalk_1.default.bold.green("[ EXTERNALS ]"));
    for (const key in externals) {
        if (externals.hasOwnProperty(key)) {
            const types = Object.keys(externals[key]);
            for (const type of types) {
                console.log(chalk_1.default.green(`[ ${key} ][ ${type} ][ ${externals[key][type]} ]`));
            }
        }
    }
    return {
        bail: !isDevelopment,
        cache: true,
        context: helpers_1.get_context(),
        devtool: isDebug ? "source-map" : false,
        entry,
        externals,
        mode: "none",
        module: {
            rules: [
                {
                    loader: "file-loader",
                    query: {
                        name: isDevelopment ? "[path][name].[ext]?[hash:8]" : "[hash:32].[ext]",
                    },
                    test: /\.(ico|jpg|jpeg|png|gif|eot|otf|webp|svg|ttf|woff|woff2)(\?.*)?$/,
                },
                {
                    test: /\.(text|csv)(\?.*)?$/,
                    use: "raw-loader",
                },
                ...css_loders_1.default(),
                ...rules,
            ],
        },
        optimization: {
            concatenateModules: false,
            flagIncludedChunks: false,
            occurrenceOrder: false,
            namedModules: isDevelopment,
            namedChunks: isDevelopment,
            nodeEnv: isDevelopment ? "development" : "production",
            minimize: !isDevelopment,
            noEmitOnErrors: !isDevelopment,
            minimizer: js_plugins_1.uglify(envConfig),
        },
        output: Object.assign({ 
            // filename - шаблон имен файлов.
            filename: isDevelopment ? "[name].js?[hash:8]" : "[chunkhash:32].js", 
            // Дописывает дополнительную информацию в bundle;
            pathinfo: isDebug, chunkFilename: isDevelopment ? "[name].chunk.js?[hash:8]" : "[chunkhash:32].chunk.js", 
            // globalObject - непонятная хрень, после того, как все отлажу, то обязательно разберусь с этой настройкой.
            globalObject: "this" }, output),
        performance: {
            hints: false,
        },
        plugins: [...plugins, new webpack.WatchIgnorePlugin([/node_modules/])],
        profile: true,
        recordsPath: path.resolve(process.cwd(), "node_modules/.cache/webpack/[confighash]/records.json"),
        resolve: {
            extensions: [".js", ".ts", ".tsx", ".css", ".json"],
            modules,
        },
        resolveLoader: {
            extensions: [".js", ".json"],
            mainFields: ["loader", "main"],
            modules,
        },
    };
}
exports.general_WP_config = general_WP_config;
// tslint:enable:variable-name object-literal-sort-keys
